/*           
Module : PSTAT.CPP
Purpose: Provides a generalised framework for executing a lengthy operation 
         in a thread. Feedback is provided by a progress dialog which can 
         optionally be cancelled.
Author : PJN / PSTAT/1 / 27-03-1997 Initially created
History: NONE

Copyright (c) 1997 PJ Naughter.
All rights reserved.

*/

///////////////////////// Includes ///////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"
#include "PStat.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif



////////////////////// Local Prototypes //////////////////////////////////////

//Use to pass a structure to the worker threads
class CProgressThreadInfo : public CObject                                                   
{
 public:
   DECLARE_DYNAMIC(CProgressThreadInfo)

   //Constructors / Destructors
   CProgressThreadInfo();

   CProgressThreadDlg*     m_pStatusDlg;        //Used to allow the UI to be updated during operation
   FUNCTION_WITH_PROGRESS* m_pfnFunction;       //Function to call to do the work
   void*                   m_pData;             //data to pass to the caller function
};


//The actual worker thread which is going to do the work
UINT ProgressDialogWorkerThread(LPVOID pParam);



IMPLEMENT_DYNAMIC(CProgressThreadInfo, CObject)


CProgressThreadInfo::CProgressThreadInfo()
{
  m_pStatusDlg = NULL;
  m_pfnFunction = NULL;
  m_pData = NULL;
}







IMPLEMENT_DYNAMIC(CProgressThreadDlg, CDialog)


CProgressThreadDlg::CProgressThreadDlg(CWnd* pParent, BOOL bShowCancelButton)
	: CDialog(bShowCancelButton ? IDD_PROGRESS_WITH_CANCEL : IDD_PROGRESS, pParent)
{
	//{{AFX_DATA_INIT(CProgressThreadDlg)
	//}}AFX_DATA_INIT
  m_nCurrentPercentage = 0;
  m_bReady = FALSE;
  m_bCancelled = FALSE;
  m_bShowCancelButton = bShowCancelButton;
  m_bOkToClose = FALSE;
}


void CProgressThreadDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CProgressThreadDlg)
  DDX_Control(pDX, IDC_PROGRESS, m_ctrlProgress);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CProgressThreadDlg, CDialog)
	//{{AFX_MSG_MAP(CProgressThreadDlg)
	ON_BN_CLICKED(IDC_CANCEL, OnCancelled)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


BOOL CProgressThreadDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

  //initialise the progress control and remember the start time
	m_ctrlProgress.SetRange(0, 100);
  m_ctrlProgress.SetPos(0);
  m_TimeCreation = CTime::GetCurrentTime();
  m_bReady = TRUE;

  if (!m_sCaption.IsEmpty())
    SetWindowText(m_sCaption);

	return TRUE;
}


void CProgressThreadDlg::OnCancel() 
{
  if (m_bOkToClose)
    CDialog::OnCancel();
  else if (m_bShowCancelButton)
    m_bCancelled = TRUE;
}


void CProgressThreadDlg::OnOK() 
{
  //deliberately do nothing
}


void CProgressThreadDlg::Close()
{
  m_bOkToClose = TRUE;
  PostMessage(WM_CLOSE);
}


void CProgressThreadDlg::OnCancelled() 
{
  OnCancel();
}


void CProgressThreadDlg::SetCaption(const CString& sCaption)
{
  m_sCaption = sCaption;
}


void CProgressThreadDlg::SetPercentageDone(int Percentage) 
{
  if (!m_bReady || m_nCurrentPercentage == Percentage)
    return;

  m_ctrlProgress.SetPos(Percentage); 
  m_nCurrentPercentage = Percentage;

  if (Percentage)  //avoid division by 0
  {
    long lSecondsLeft = ((CTime::GetCurrentTime() - m_TimeCreation).GetTotalSeconds() * (100 - Percentage)) / Percentage;
    CTimeSpan TimeLeft = CTimeSpan(lSecondsLeft);

    int Minutes = TimeLeft.GetMinutes();
    int Seconds = TimeLeft.GetSeconds();

    CString sVal;
    CString sText;
    if (Minutes)
    {
      sVal.Format(_T("%d"), Minutes);
      AfxFormatString1(sText, IDS_PSTAT_TIMELEFT_IN_MINUTES, sVal);
    }
    else
    {
      if (Seconds)
      {
        sVal.Format(_T("%d"), Seconds);
        AfxFormatString1(sText, IDS_PSTAT_TIMELEFT_IN_SECONDS, sVal);
      }
      else 
      {
        //if 0 seconds are left then display no text
        sText = CString(_T(""));
      }
    }

    ASSERT(GetDlgItem(IDC_TIMELEFT));
    GetDlgItem(IDC_TIMELEFT)->SetWindowText(sText);
  }
}



BOOL ExecuteFunctionWithProgressDialog(FUNCTION_WITH_PROGRESS* pfnFunction, const CString& sProgressTitle, 
                                       void* pData, DWORD dwFlags, int nPriority, CWnd* pParent)
{
  BOOL bShowCancel = (dwFlags & PSTAT_CANCEL);

  CProgressThreadDlg dlg(pParent, bShowCancel);
  dlg.SetCaption(sProgressTitle);
  CProgressThreadInfo Info;
  Info.m_pStatusDlg = &dlg;
  Info.m_pfnFunction = pfnFunction;
  Info.m_pData = pData;
  CWinThread* pThread = AfxBeginThread(ProgressDialogWorkerThread, &Info, nPriority);

  //If the thread failed to create itself
  if (!pThread)
  {
    ASSERT(FALSE);
    return FALSE;
  }

  //bring up the dialog modal (thread will close it for us)
  dlg.DoModal();

  return !dlg.HasBeenCancelled();
}


UINT ProgressDialogWorkerThread(LPVOID pParam)
{
  CProgressThreadInfo* pInfo = (CProgressThreadInfo*) pParam;
  ASSERT(pInfo);
  ASSERT(pInfo->IsKindOf(RUNTIME_CLASS(CProgressThreadInfo)));

  //Call the user defined function
  pInfo->m_pfnFunction(pInfo->m_pData, pInfo->m_pStatusDlg);

  //close down the progress dialog
  pInfo->m_pStatusDlg->Close();

  return 0;
}

